==> org.eclipse.elk.alg.layered.source_0.1.0.201607121925.jar: org/eclipse/elk/alg/layered/intermediate/PortSideProcessor.java <==
/*******************************************************************************
 * Copyright (c) 2011, 2015 Kiel University and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Kiel University - initial API and implementation
 *******************************************************************************/
package org.eclipse.elk.alg.layered.intermediate;

import org.eclipse.elk.alg.layered.ILayoutProcessor;
import org.eclipse.elk.alg.layered.graph.LGraph;
import org.eclipse.elk.alg.layered.graph.LNode;
import org.eclipse.elk.alg.layered.graph.LPort;
import org.eclipse.elk.alg.layered.graph.Layer;
import org.eclipse.elk.alg.layered.properties.LayeredOptions;
import org.eclipse.elk.core.options.PortConstraints;
import org.eclipse.elk.core.options.PortSide;
import org.eclipse.elk.core.util.IElkProgressMonitor;

/**
 * Makes sure ports have at least a fixed side. If they don't, input ports are assigned
 * to the left and output ports to the right side. This processor can run either before the
 * first phase or before the third phase. In the first slot it assigns port sides before any
 * edges are reversed, hence edges that are reversed later are routed "around" the nodes
 * using inverted ports. This can only be handled correctly if the {@link InvertedPortProcessor}
 * is active. In the third slot, however, the port sides are assigned <em>after</em> edges
 * are reversed, so no inverted ports will occur. This behavior is controlled by the option
 * {@link org.eclipse.elk.alg.layered.properties.LayeredOptions#FEEDBACK_EDGES FEEDBACK_EDGES}.
 * 
 * <dl>
 *   <dt>Precondition:</dt><dd>a layered graph.</dd>
 *   <dt>Postcondition:</dt><dd>all nodes have their ports distributed, with port constraints
 *     set to fixed sides at the least.</dd>
 *   <dt>Slots:</dt><dd>Before phase 1 or before phase 3.</dd>
 *   <dt>Same-slot dependencies:</dt><dd>None.</dd>
 * </dl>
 * 
 * @author cds
 * @kieler.design 2012-08-10 chsch grh
 * @kieler.rating proposed yellow by msp
 */
public final class PortSideProcessor implements ILayoutProcessor {

    /**
     * {@inheritDoc}
     */
    public void process(final LGraph layeredGraph, final IElkProgressMonitor monitor) {
        monitor.begin("Port side processing", 1);
        
        // IF USED BEFORE PHASE 1
        // Iterate through the unlayered nodes of the graph
        for (LNode node : layeredGraph.getLayerlessNodes()) {
            process(node);
        }
        
        // IF USED BEFORE PHASE 3
        // Iterate through the nodes of all layers
        for (Layer layer : layeredGraph) {
            for (LNode node : layer) {
                process(node);
            }
        }
        
        monitor.done();
    }
    
    /**
     * Process the ports of the given node.
     * 
     * @param node a node
     */
    private void process(final LNode node) {
        if (node.getProperty(LayeredOptions.PORT_CONSTRAINTS).isSideFixed()) {
            // Check whether there are ports with undefined side
            for (LPort port : node.getPorts()) {
                if (port.getSide() == PortSide.UNDEFINED) {
                    setPortSide(port);
                }
            }
        } else {
            // Distribute all ports and change constraints to fixed side
            for (LPort port : node.getPorts()) {
                setPortSide(port);
            }
            node.setProperty(LayeredOptions.PORT_CONSTRAINTS, PortConstraints.FIXED_SIDE);
        }
    }
    
    /**
     * Places input ports on the left side and output ports on the right side of nodes.
     * 
     * @param port the port to set side and anchor position
     */
    public static void setPortSide(final LPort port) {
        if (port.getNetFlow() < 0) {
            port.setSide(PortSide.EAST);
            // adapt the anchor so outgoing edges are attached right
            port.getAnchor().x = port.getSize().x;
        } else {
            port.setSide(PortSide.WEST);
            // adapt the anchor so incoming edges are attached left
            port.getAnchor().x = 0;
        }
    }

}

==> org.eclipse.elk.alg.layered.source_0.5.0.jar: org/eclipse/elk/alg/layered/intermediate/PortSideProcessor.java <==
/*******************************************************************************
 * Copyright (c) 2011, 2015 Kiel University and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Kiel University - initial API and implementation
 *******************************************************************************/
package org.eclipse.elk.alg.layered.intermediate;

import org.eclipse.elk.alg.layered.graph.LGraph;
import org.eclipse.elk.alg.layered.graph.LNode;
import org.eclipse.elk.alg.layered.graph.LPort;
import org.eclipse.elk.alg.layered.graph.Layer;
import org.eclipse.elk.alg.layered.options.InternalProperties;
import org.eclipse.elk.alg.layered.options.LayeredOptions;
import org.eclipse.elk.core.alg.ILayoutProcessor;
import org.eclipse.elk.core.math.KVector;
import org.eclipse.elk.core.options.PortConstraints;
import org.eclipse.elk.core.options.PortSide;
import org.eclipse.elk.core.util.IElkProgressMonitor;

/**
 * Makes sure ports have at least a fixed side. If they don't, input ports are assigned
 * to the left and output ports to the right side. This processor can run either before the
 * first phase or before the third phase. In the first slot it assigns port sides before any
 * edges are reversed, hence edges that are reversed later are routed "around" the nodes
 * using inverted ports. This can only be handled correctly if the {@link InvertedPortProcessor}
 * is active. In the third slot, however, the port sides are assigned <em>after</em> edges
 * are reversed, so no inverted ports will occur. This behavior is controlled by the option
 * {@link org.eclipse.elk.alg.layered.options.LayeredOptions#FEEDBACK_EDGES FEEDBACK_EDGES}.
 * 
 * <dl>
 *   <dt>Precondition:</dt><dd>a layered graph.</dd>
 *   <dt>Postcondition:</dt><dd>all nodes have their ports distributed, with port constraints
 *     set to fixed sides at the least.</dd>
 *   <dt>Slots:</dt><dd>Before phase 1 or before phase 3.</dd>
 *   <dt>Same-slot dependencies:</dt><dd>None.</dd>
 * </dl>
 * 
 * @author cds
 */
public final class PortSideProcessor implements ILayoutProcessor<LGraph> {

    /**
     * {@inheritDoc}
     */
    public void process(final LGraph layeredGraph, final IElkProgressMonitor monitor) {
        monitor.begin("Port side processing", 1);
        
        // IF USED BEFORE PHASE 1
        // Iterate through the unlayered nodes of the graph
        for (LNode node : layeredGraph.getLayerlessNodes()) {
            process(node);
        }
        
        // IF USED BEFORE PHASE 3
        // Iterate through the nodes of all layers
        for (Layer layer : layeredGraph) {
            for (LNode node : layer) {
                process(node);
            }
        }
        
        monitor.done();
    }
    
    /**
     * Process the ports of the given node.
     * 
     * @param node a node
     */
    private void process(final LNode node) {
        if (node.getProperty(LayeredOptions.PORT_CONSTRAINTS).isSideFixed()) {
            // Check whether there are ports with undefined side
            for (LPort port : node.getPorts()) {
                if (port.getSide() == PortSide.UNDEFINED) {
                    setPortSide(port);
                }
            }
        } else {
            // Distribute all ports and change constraints to fixed side
            for (LPort port : node.getPorts()) {
                setPortSide(port);
            }
            node.setProperty(LayeredOptions.PORT_CONSTRAINTS, PortConstraints.FIXED_SIDE);
        }
    }
    
    /**
     * Places input ports on the left side and output ports on the right side of nodes.
     * 
     * @param port the port to set side and anchor position
     */
    public static void setPortSide(final LPort port) {
        LNode portDummy = port.getProperty(InternalProperties.PORT_DUMMY);
        if (portDummy != null) {
            port.setSide(portDummy.getProperty(InternalProperties.EXT_PORT_SIDE));
        } else if (port.getNetFlow() < 0) {
            port.setSide(PortSide.EAST);
        } else {
            port.setSide(PortSide.WEST);
        }
        
        // If the port anchor is not fixed, adapt it to its port side
        if (!port.isAnchorFixed()) {
            KVector portSize = port.getSize();
            KVector portAnchor = port.getAnchor();
            
            switch (port.getSide()) {
            case NORTH:
                portAnchor.x = portSize.x / 2;
                portAnchor.y = 0;
                break;
            case EAST:
                portAnchor.x = portSize.x;
                portAnchor.y = portSize.y / 2;
                break;
            case SOUTH:
                portAnchor.x = portSize.x / 2;
                portAnchor.y = portSize.y;
                break;
            case WEST:
                portAnchor.x = 0;
                portAnchor.y = portSize.y / 2;
                break;
            }
        }
    }

}
